Option Explicit On
Option Strict On
Option Compare Binary

Imports System
Imports System.Drawing
Imports System.Threading
Imports System.Windows.Forms
Imports System.ComponentModel

<ToolboxItem(True)> _
Public Class uclFontListbox
    Inherits System.Windows.Forms.UserControl

    Private Shared FontStyles As FontStyle() = _
        DirectCast([Enum].GetValues(GetType(FontStyle)), FontStyle())

    Private Shared _FontFamilies As FontFamily() = FontFamily.Families

    Private _ItemFontSize As Single
    Private _DataSource(-1) As Font

#Region " Vom Windows Form Designer generierter Code "

    Private components As System.ComponentModel.IContainer

    Friend WithEvents txtCurrentFont As System.Windows.Forms.TextBox
    Friend WithEvents lblSelectedFontName As System.Windows.Forms.Label

    <System.Diagnostics.DebuggerStepThrough()> _
    Private Sub InitializeComponent()
        Me.txtCurrentFont = New System.Windows.Forms.TextBox
        Me.lblSelectedFontName = New System.Windows.Forms.Label
        Me.ListBox1 = New System.Windows.Forms.ListBox
        Me.SuspendLayout()
        '
        'txtCurrentFont
        '
        Me.txtCurrentFont.Dock = System.Windows.Forms.DockStyle.Top
        Me.txtCurrentFont.Location = New System.Drawing.Point(0, 0)
        Me.txtCurrentFont.Name = "txtCurrentFont"
        Me.txtCurrentFont.Size = New System.Drawing.Size(177, 21)
        Me.txtCurrentFont.TabIndex = 0
        '
        'lblSelectedFontName
        '
        Me.lblSelectedFontName.Dock = System.Windows.Forms.DockStyle.Bottom
        Me.lblSelectedFontName.Location = New System.Drawing.Point(0, 216)
        Me.lblSelectedFontName.Name = "lblSelectedFontName"
        Me.lblSelectedFontName.Size = New System.Drawing.Size(177, 28)
        Me.lblSelectedFontName.TabIndex = 2
        Me.lblSelectedFontName.Text = "#"
        Me.lblSelectedFontName.TextAlign = System.Drawing.ContentAlignment.MiddleLeft
        '
        'ListBox1
        '
        Me.ListBox1.Dock = System.Windows.Forms.DockStyle.Fill
        Me.ListBox1.DrawMode = System.Windows.Forms.DrawMode.OwnerDrawVariable
        Me.ListBox1.Location = New System.Drawing.Point(0, 21)
        Me.ListBox1.Name = "ListBox1"
        Me.ListBox1.Size = New System.Drawing.Size(177, 195)
        Me.ListBox1.TabIndex = 3
        '
        'uclFontListbox
        '
        Me.Controls.Add(Me.ListBox1)
        Me.Controls.Add(Me.lblSelectedFontName)
        Me.Controls.Add(Me.txtCurrentFont)
        Me.Font = New System.Drawing.Font("Tahoma", 8.25!, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, CType(0, Byte))
        Me.Name = "uclFontListbox"
        Me.Size = New System.Drawing.Size(177, 244)
        Me.ResumeLayout(False)
        Me.PerformLayout()

    End Sub
    Friend WithEvents ListBox1 As System.Windows.Forms.ListBox
#End Region

#Region "Init"

    Public Sub New()
        ReDim _DataSource(_FontFamilies.Length - 1)
        InitializeComponent()
        Me.lblSelectedFontName.DataBindings.Add("Text", _DataSource, "Name")
        Me.ItemFontSize = 12
    End Sub

#End Region 'Init

#Region "Events & Overrides"

    Private Sub txtCurrentFont_TextChanged(ByVal sender As Object, ByVal e As EventArgs) _
            Handles txtCurrentFont.TextChanged
        FindSimilar(txtCurrentFont.Text)
    End Sub

    Private Sub ListBox1_DrawItem(ByVal sender As Object, ByVal e As DrawItemEventArgs) _
            Handles ListBox1.DrawItem
        Dim fntCurrent As Font = _DataSource(e.Index)
        Dim DrawBrush As Brush
        e.DrawBackground()
        If (e.State And DrawItemState.Selected) = DrawItemState.Selected Then
            DrawBrush = SystemBrushes.HighlightText
        Else
            DrawBrush = SystemBrushes.WindowText
        End If
        e.Graphics.DrawString( _
            fntCurrent.Name, _
            fntCurrent, _
            DrawBrush, _
            e.Bounds.Left, _
            e.Bounds.Top + 1)
        If (e.State And DrawItemState.Focus) = DrawItemState.Focus Then
            e.DrawFocusRectangle()
        End If
    End Sub

    Private Sub ListBox1_MeasureItem(ByVal sender As Object, ByVal e As MeasureItemEventArgs) _
          Handles ListBox1.MeasureItem
        Dim fntCurrent As Font = _DataSource(e.Index)
        With e.Graphics.MeasureString(fntCurrent.Name, fntCurrent)
            e.ItemHeight = CInt(.Height) + 2
            e.ItemWidth = CInt(.Width)
        End With
    End Sub

    Protected Overloads Overrides Sub Dispose(ByVal disposing As Boolean)
        If disposing Then
            If Not (components Is Nothing) Then
                DisposeFonts()
                components.Dispose()
            End If
        End If
        MyBase.Dispose(disposing)
    End Sub

#End Region 'Events & Overrides

#Region "Publix"

    Public ReadOnly Property SelectedFont() As Font
        Get
            Return DirectCast(ListBox1.SelectedItem, Font)
        End Get
    End Property

    <Bindable(True), DefaultValue(12.0!), Category("Darstellung")> _
    Public Property ItemFontSize() As Single
        Get
            Return _ItemFontSize
        End Get
        Set(ByVal Value As Single)
            If _ItemFontSize = Value Then Return
            _ItemFontSize = Value
            Refill()
        End Set
    End Property

#End Region 'Publix

#Region "Privates"

    Private Sub DisposeFonts()
        For Each Ft As Font In _DataSource
            Ft.Dispose()
        Next
    End Sub

    Private Sub Refill()
        If Not (_DataSource(0) Is Nothing) Then
            DisposeFonts()
        End If
        For I As Integer = 0 To _DataSource.Length - 1
            For Each Style As FontStyle In FontStyles
                Dim fntFam As FontFamily = _FontFamilies(I)
                If fntFam.IsStyleAvailable(Style) Then
                    Dim Ft As New Font(fntFam, _ItemFontSize, Style, GraphicsUnit.Pixel)
                    _DataSource(I) = Ft
                    Exit For
                End If
            Next
        Next
        Me.ListBox1.DataSource = Nothing
        Me.ListBox1.DataSource = _DataSource
    End Sub

    ''' <summary>
    ''' selektiert den Font, dessen Name mit dem Pattern beginnt,
    ''' bei NichtFinden eines Matches den davor liegenden
    ''' </summary>
    Private Sub FindSimilar(ByVal Pattern As String)
        For I As Integer = 0 To _DataSource.Length - 1
            Select Case String.Compare( _
                    DirectCast(_DataSource(I), Font).Name, _
                    0, Pattern, 0, Pattern.Length, ignoreCase:=True)
                Case Is < 0
                Case 0
                    ListBox1.TopIndex = I
                    ListBox1.SelectedIndex = I
                    Return
                Case Is > 0
                    If I > 0 Then
                        ListBox1.TopIndex = I - 1
                        ListBox1.SelectedIndex = I - 1
                    Else
                        ListBox1.TopIndex = I
                        ListBox1.SelectedIndex = I
                    End If
                    Return
            End Select
        Next
    End Sub

#End Region 'Privates

End Class